/* Copyright 2007-2015 QReal Research Group
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. */

#include "errorListWidget.h"

#include <QtWidgets/QApplication>
#include <QtWidgets/QLabel>
#include <QtWidgets/QMenu>
#include <QtGui/QTextDocument>
#include <QtGui/QClipboard>

#include "mainWindow/mainWindow.h"
#include "qrkernel/settingsManager.h"

ErrorListWidget::ErrorListWidget(QWidget *parent)
	: QListWidget(parent)
{
	connect(this, &ErrorListWidget::itemDoubleClicked, this, &ErrorListWidget::highlightElement);

	bool ok;
	auto size = qReal::SettingsManager::value("CustomDockTextSize").toInt(&ok);
	if (ok) {
		auto f = font();
		f.setPointSize(size);
		setFont(f);
	}

	auto height = fontMetrics().boundingRect("'_").height();
	setIconSize(QSize(height, height));
	initContextMenu();
}

void ErrorListWidget::highlightElement(QListWidgetItem* const item)
{
	const qReal::Id id = qReal::Id::loadFromString(item->data(positionRole).toString());
	if (item->isSelected()) {
		mMainWindow->selectItemWithError(id);
	}
}

void ErrorListWidget::init(qReal::MainWindow* mainWindow)
{
	mMainWindow = mainWindow;
}

void ErrorListWidget::initContextMenu()
{
	mContextMenu = new QMenu(this);
	QAction *clearAction = mContextMenu->addAction(tr("Clear"));
	QAction *copyAction = mContextMenu->addAction(tr("Copy"));
	connect(clearAction, &QAction::triggered, this, &ErrorListWidget::clearRequested);
	connect(copyAction, &QAction::triggered, this, &ErrorListWidget::copyCurrentItem);

	setContextMenuPolicy(Qt::CustomContextMenu);
	connect(this, &ErrorListWidget::customContextMenuRequested, this, &ErrorListWidget::showContextMenu);
}

void ErrorListWidget::showContextMenu(const QPoint &pos)
{
	mContextMenu->exec(mapToGlobal(pos));
}

void ErrorListWidget::copyCurrentItem()
{
	QListWidgetItem *item = currentItem();
	QLabel *label = item ? dynamic_cast<QLabel *>(itemWidget(item)) : nullptr;
	if (label) {
		// Extracting a plain text
		QTextDocument document;
		document.setHtml(label->text());
		QApplication::clipboard()->setText(document.toPlainText());
	}
}
